/**
 * This class instantiates a complex Infix Calculator that reads input from
 * standard in and then computes and displays the answer or prints
 * an error message.
 *
 * @author <A HREF="mailto:gte220j@prism.gatech.edu.com"> J.Chris Holmes </A>
 * @version Version 1.0, 4/23/2001
 */

import java.util.StringTokenizer;
import java.io.IOException;

public class Infix_Calc2
{
  /**
   * Default constructor for a complex Infix Calculator object
   */
  public Infix_Calc2() {
    boolean not_done = true;
    String input_string; 
    int output;

    while( not_done == true) {
      try {
        input_string = IOHelp.readLine();

        if(input_string.compareToIgnoreCase("q") == 0) 
          not_done = false;
        else { 
          try {
            output = RunCalculation(input_string);
            System.out.println("ans = "+output);
          }
          catch(InvalidInputException iie) {
            System.out.println(iie.getMessage());
          }
        }
      }
      catch(IOException ioe) {
        ioe.printStackTrace();
        not_done = false;
      }
    }
  }
  /**
   * Takes in a String of math in Infix Notation form (allows parentheses) and
   * returns the answer or throws an Exception.
   *
   * @param input_string  A string of math in Infix Notation form
   * @return  An int of the result
   */
  public int RunCalculation(String input_string) throws InvalidInputException {
    PushBackStringTokenizer strtok;
    String token1, token2, pulledout;
    int op1, op2, parencount;

    strtok = new PushBackStringTokenizer(input_string, " \t");

    while(strtok.hasMoreTokens()) {
      token1 = strtok.nextToken();
      
      if( isNum(token1) == true) {
        op1 = Integer.parseInt(token1);
      }
      else if( isOpenParen(token1) == true) {
        parencount = 1;
        pulledout = "";
        while(parencount > 0) {
          if(strtok.hasMoreTokens() == false) 
            throw new InvalidInputException("ERROR:  Expression Invalid");
          
          token1 = strtok.nextToken();
          if( isOpenParen(token1) == true) { parencount++; }
          else if( isCloseParen(token1) == true) { parencount--; }

          if(parencount != 0)  // don't push on the last close parenthesis
            pulledout += (token1 + " ");
        }
        op1 = RunCalculation(pulledout); 
      }
      else {
        throw new InvalidInputException("ERROR:  Expression Invalid");
      }
// op1 is guaranteed to have valid data in it now

      // It is entirely possible that the only operand left has been loaded into op1
      // If this is true, then we're done, just return it
      if(strtok.countTokens() == 0) {
        return(op1);
      }

      // So we have at least 1 token left, make sure we have at least 2, otherwise
      // this is an invalid string.
      
      if(strtok.countTokens() < 2) 
        throw new InvalidInputException("ERROR:  Expression Invalid");  


      token1 = strtok.nextToken();

      if(isOperator(token1) == false) 
        throw new InvalidInputException("ERROR:  Expression Invalid");  

// operator is now fetched and being held in token1

      token2 = strtok.nextToken();

      if( isNum(token2) == true) {
        op2 = Integer.parseInt(token2);
      }
      else if( isOpenParen(token2) == true) {
        parencount = 1;
        pulledout = "";
        while(parencount > 0) {
          if(strtok.hasMoreTokens() == false) 
            throw new InvalidInputException("ERROR:  Expression Invalid");
          
          token2 = strtok.nextToken();
          if( isOpenParen(token2) == true) { parencount++; }
          else if( isCloseParen(token2) == true) { parencount--; }
         
          if(parencount != 0)  // don't push on the last close parenthesis
            pulledout += (token2 + " ");
        }
        op2 = RunCalculation(pulledout); 
      }
      else {
        throw new InvalidInputException("ERROR:  Expression Invalid");
      }
// op1 and op2 now hold ints and token1 holds the operator string

      switch(token1.charAt(0)) {
        case '+': 
          op1 = op1 + op2;
         break;
        case '-': 
          op1 = op1 - op2;          
         break;
        case '*': 
          op1 = op1 * op2;
         break;
        case '/': 
          op1 = op1 / op2;
         break;
        case '^': 
          op1 = (int)Math.pow((double)op1,(double)op2);
         break;
        default: 
         break;
      }

// shove the answer back onto the tokenizer so it can be re-extracted on the next pass

      strtok.PushBack(""+op1);

      if(strtok.countTokens() < 3) {
        if(strtok.countTokens() == 1) {
          break;
        }
        else 
          throw new InvalidInputException("ERROR:  Expression Invalid");      
      }
    }
    return(Integer.parseInt(strtok.nextToken()));
  } 
  /**
   * Takes in a single string token and returns a boolean based on
   * whether or not the string is a valid number
   *
   * @param to_check  Any String
   * @return  A boolean that is true is the input String is a valid number
   */
  public boolean isNum(String to_check) {
    boolean toret = false;
    try {
      Integer.parseInt(to_check);
      toret = true;
    }
    catch(NumberFormatException nfe) {
      toret = false;
    }
    return(toret);
  }
  /**
   * Takes in a single string token and returns a boolean based on
   * whether or not the string is a valid mathematical operator
   *
   * @param to_check  Any String
   * @return  A boolean that is true is the input String is a valid operator
   */
  public boolean isOperator(String to_check) {
    if(to_check == null) return false;
    if(to_check.length() != 1) return false; 

    boolean toret = false;
    switch(to_check.charAt(0)) {
      case '+': 
      case '-':
      case '*':
      case '/':
      case '^':
        toret = true; break;
      default: break;
    }
    return(toret);
  }
  /**
   * Takes in a single string token and returns a boolean based on
   * whether or not the string is an open parenthesis '('
   *
   * @param to_check  Any String
   * @return  A boolean that is true is the input String is an open parenthesis
   */
  public boolean isOpenParen(String to_check) {
    if(to_check == null) return false;
    if(to_check.length() != 1) return false; 
    if(to_check.charAt(0) == '(') return true;
    return false;
  }
  /**
   * Takes in a single string token and returns a boolean based on
   * whether or not the string is an close parenthesis ')'
   *
   * @param to_check  Any String
   * @return  A boolean that is true is the input String is an close parenthesis
   */
  public boolean isCloseParen(String to_check) {
    if(to_check == null) return false;
    if(to_check.length() != 1) return false; 
    if(to_check.charAt(0) == ')') return true;
    return false;
  }
  /**
   * Main method -- instantiates a Calculator
   *
   * @param args  Command line arguements
   */
  public static void main(String args[]) {
    new Infix_Calc2();
  }
}