/**
 * This class adds the ability to push data back onto a StringTokenizer.
 *
 * @author <A HREF="mailto:gte220j@prism.gatech.edu.com"> J.Chris Holmes </A>
 * @version Version 1.0, 4/23/2001
 */

import java.util.StringTokenizer;

public class PushBackStringTokenizer extends StringTokenizer
{
  private Queue PushBackBuffer;

  /**
   * Chains to a similar constructor in the StringTokenizer
   */
  public PushBackStringTokenizer(String str) {
    super(str);
    PushBackBuffer = new Queue();
  }

  /**
   * Chains to a similar constructor in the StringTokenizer
   */
  public PushBackStringTokenizer(String str, String delim) {
    super(str, delim);
    PushBackBuffer = new Queue();
  }

  /**
   * Chains to a similar constructor in the StringTokenizer
   */
  public PushBackStringTokenizer(String str, String delim, boolean returnDelims) {
    super(str, delim, returnDelims);
    PushBackBuffer = new Queue();
  }

  /**
   * Pushes a String back into the StringTokenizer
   *
   * @param s  The String to push back onto the buffer
   */
  public void PushBack(String s) {
    PushBackBuffer.enqueue(s);
  }

  /**
   * Returns the next available String from the Tokenizer
   *
   * @returns  The next available String from the Tokenizer
   */
  public String nextToken() {
    if(PushBackBuffer.isEmpty()) {
      return(super.nextToken());
    }
    else {
      return((String)PushBackBuffer.dequeue());
    }
  }
  /**
   * Returns true or false based on whether the Tokenizer has
   * more tokens.
   *
   * @returns  True if there are more tokens in the Tokenizer
   */
  public boolean hasMoreTokens() {
    if(PushBackBuffer.isEmpty() == true) {
      return(super.hasMoreTokens());
    }
    else { return(true); }
  }

  /**
   * Returns the number of available tokens
   *
   * @returns  The number of available tokens
   */
  public int countTokens() {
    return(PushBackBuffer.size() + super.countTokens());
  }
}
